@@ -1,5 +1,5 @@
 #-----------------------------------------------------------------------------
-# Copyright (c) 2013-2021, PyInstaller Development Team.
+# Copyright (c) 2013-2023, PyInstaller Development Team.
 #
 # Distributed under the terms of the GNU General Public License (version 2
 # or later) with exception for distributing the bootloader.
@@ -8,31 +8,30 @@
 #
 # SPDX-License-Identifier: (GPL-2.0-or-later WITH Bootloader-exception)
 #-----------------------------------------------------------------------------
-
-
 """
 This CArchiveReader is used only by the archieve_viewer utility.
 """
 
 # TODO clean up this module
 
+import os
 import struct
 
-
-from PyInstaller.loader.pyimod02_archive import ArchiveReader
+from PyInstaller.loader.pyimod01_archive import ArchiveReader
 
 
 class NotAnArchiveError(Exception):
     pass
 
 
-class CTOCReader(object):
+class CTOCReader:
     """
     A class encapsulating the table of contents of a CArchive.
 
     When written to disk, it is easily read from C.
     """
-    ENTRYSTRUCT = '!iiiiBB'  # (structlen, dpos, dlen, ulen, flag, typcd) followed by name
+    # (structlen, dpos, dlen, ulen, flag, typcd) followed by name
+    ENTRYSTRUCT = '!iIIIBB'
     ENTRYLEN = struct.calcsize(ENTRYSTRUCT)
 
     def __init__(self):
@@ -47,11 +46,10 @@ class CTOCReader(object):
         p = 0
 
         while p < len(s):
-            (slen, dpos, dlen, ulen, flag, typcd) = struct.unpack(self.ENTRYSTRUCT,
-                                                        s[p:p + self.ENTRYLEN])
+            slen, dpos, dlen, ulen, flag, typcd = struct.unpack(self.ENTRYSTRUCT, s[p:p + self.ENTRYLEN])
             nmlen = slen - self.ENTRYLEN
             p = p + self.ENTRYLEN
-            (nm,) = struct.unpack('%is' % nmlen, s[p:p + nmlen])
+            nm, = struct.unpack('%is' % nmlen, s[p:p + nmlen])
             p = p + nmlen
             # nm may have up to 15 bytes of padding
             nm = nm.rstrip(b'\0')
@@ -59,7 +57,6 @@ class CTOCReader(object):
             typcd = chr(typcd)
             self.data.append((dpos, dlen, ulen, flag, typcd, nm))
 
-
     def get(self, ndx):
         """
         Return the table of contents entry (tuple) at index NDX.
@@ -85,32 +82,29 @@ class CArchiveReader(ArchiveReader):
     """
     An Archive subclass that can hold arbitrary data.
 
-    This class encapsulates all files that are bundled within an executable.
-    It can contain ZlibArchive (Python .pyc files), dlls, Python C extensions
-    and all other data files that are bundled in --onefile mode.
+    This class encapsulates all files that are bundled within an executable. It can contain ZlibArchive (Python .pyc
+    files), dlls, Python C extensions and all other data files that are bundled in --onefile mode.
 
     Easily handled from C or from Python.
     """
-    # MAGIC is useful to verify that conversion of Python data types
-    # to C structure and back works properly.
+    # MAGIC is useful to verify that conversion of Python data types to C structure and back works properly.
     MAGIC = b'MEI\014\013\012\013\016'
     HDRLEN = 0
     LEVEL = 9
 
-    # Cookie - holds some information for the bootloader. C struct format
-    # definition. '!' at the beginning means network byte order.
-    # C struct looks like:
+    # Cookie - holds some information for the bootloader. C struct format definition. '!' at the beginning means network
+    # byte order. C struct looks like:
     #
     #   typedef struct _cookie {
     #       char magic[8]; /* 'MEI\014\013\012\013\016' */
-    #       int  len;      /* len of entire package */
-    #       int  TOC;      /* pos (rel to start) of TableOfContents */
+    #       uint32_t len;  /* len of entire package */
+    #       uint32_t TOC;  /* pos (rel to start) of TableOfContents */
     #       int  TOClen;   /* length of TableOfContents */
     #       int  pyvers;   /* new in v4 */
     #       char pylibname[64];    /* Filename of Python dynamic library. */
     #   } COOKIE;
     #
-    _cookie_format = '!8siiii64s'
+    _cookie_format = '!8sIIii64s'
     _cookie_size = struct.calcsize(_cookie_format)
 
     def __init__(self, archive_path=None, start=0, length=0, pylib_name=''):
@@ -125,10 +119,9 @@ class CArchiveReader(ArchiveReader):
         self.length = length
         self._pylib_name = pylib_name
 
-
         # A CArchive created from scratch starts at 0, no leading bootloader.
         self.pkg_start = 0
-        super(CArchiveReader, self).__init__(archive_path, start)
+        super().__init__(archive_path, start)
 
     def checkmagic(self):
         """
@@ -136,34 +129,46 @@ class CArchiveReader(ArchiveReader):
 
         Magic signature is at end of the archive.
 
-        This fuction is used by ArchiveViewer.py utility.
+        This function is used by ArchiveViewer.py utility.
         """
         # Magic is at EOF; if we're embedded, we need to figure where that is.
         if self.length:
             self.lib.seek(self.start + self.length, 0)
         else:
-            self.lib.seek(0, 2)
-        filelen = self.lib.tell()
+            self.lib.seek(0, os.SEEK_END)
+        end_pos = self.lib.tell()
 
-        self.lib.seek(max(0, filelen-4096))
-        searchpos = self.lib.tell()
-        buf = self.lib.read(min(filelen, 4096))
-        pos = buf.rfind(self.MAGIC)
-        if pos == -1:
-            raise RuntimeError("%s is not a valid %s archive file" %
-                               (self.path, self.__class__.__name__))
-        filelen = searchpos + pos + self._cookie_size
-        (magic, totallen, tocpos, toclen, pyvers, pylib_name) = struct.unpack(
-            self._cookie_format, buf[pos:pos+self._cookie_size])
+        SEARCH_CHUNK_SIZE = 8192
+        magic_offset = -1
+        while end_pos >= len(self.MAGIC):
+            start_pos = max(end_pos - SEARCH_CHUNK_SIZE, 0)
+            chunk_size = end_pos - start_pos
+            # Is the remaining chunk large enough to hold the pattern?
+            if chunk_size < len(self.MAGIC):
+                break
+            # Read and scan the chunk
+            self.lib.seek(start_pos, os.SEEK_SET)
+            buf = self.lib.read(chunk_size)
+            pos = buf.rfind(self.MAGIC)
+            if pos != -1:
+                magic_offset = start_pos + pos
+                break
+            # Adjust search location for next chunk; ensure proper overlap
+            end_pos = start_pos + len(self.MAGIC) - 1
+        if magic_offset == -1:
+            raise RuntimeError("%s is not a valid %s archive file" % (self.path, self.__class__.__name__))
+        filelen = magic_offset + self._cookie_size
+        # Read the whole cookie
+        self.lib.seek(magic_offset, os.SEEK_SET)
+        buf = self.lib.read(self._cookie_size)
+        magic, totallen, tocpos, toclen, pyvers, pylib_name = struct.unpack(self._cookie_format, buf)
         if magic != self.MAGIC:
-            raise RuntimeError("%s is not a valid %s archive file" %
-                               (self.path, self.__class__.__name__))
+            raise RuntimeError("%s is not a valid %s archive file" % (self.path, self.__class__.__name__))
 
         self.pkg_start = filelen - totallen
         if self.length:
             if totallen != self.length or self.pkg_start != self.start:
-                raise RuntimeError('Problem with embedded archive in %s' %
-                        self.path)
+                raise RuntimeError('Problem with embedded archive in %s' % self.path)
         # Verify presence of Python library name.
         if not pylib_name:
             raise RuntimeError('Python library filename not defined in archive.')
@@ -185,7 +190,7 @@ class CArchiveReader(ArchiveReader):
         NAME is an entry name OR the index to the TOC.
 
         Return the tuple (ispkg, contents).
-        For non-Python resoures, ispkg is meaningless (and 0).
+        For non-Python resources, ispkg is meaningless (and 0).
         Used by the import mechanism.
         """
         if isinstance(name, str):
@@ -194,7 +199,7 @@ class CArchiveReader(ArchiveReader):
                 return None
         else:
             ndx = name
-        (dpos, dlen, ulen, flag, typcd, nm) = self.toc.get(ndx)
+        dpos, dlen, ulen, flag, typcd, nm = self.toc.get(ndx)
 
         with self.lib:
             self.lib.seek(self.pkg_start + dpos)
@@ -204,16 +209,16 @@ class CArchiveReader(ArchiveReader):
             import zlib
             rslt = zlib.decompress(rslt)
         if typcd == 'M':
-            return (1, rslt)
+            return 1, rslt
 
-        return (typcd == 'M', rslt)
+        return typcd == 'M', rslt
 
     def contents(self):
         """
         Return the names of the entries.
         """
         rslt = []
-        for (dpos, dlen, ulen, flag, typcd, nm) in self.toc:
+        for dpos, dlen, ulen, flag, typcd, nm in self.toc:
             rslt.append(nm)
         return rslt
 
@@ -221,18 +226,17 @@ class CArchiveReader(ArchiveReader):
         """
         Open a CArchive of name NAME embedded within this CArchive.
 
-        This fuction is used by ArchiveViewer.py utility.
+        This function is used by ArchiveViewer.py utility.
         """
         ndx = self.toc.find(name)
 
         if ndx == -1:
             raise KeyError("Member '%s' not found in %s" % (name, self.path))
-        (dpos, dlen, ulen, flag, typcd, nm) = self.toc.get(ndx)
+        dpos, dlen, ulen, flag, typcd, nm = self.toc.get(ndx)
 
         if typcd not in "zZ":
             raise NotAnArchiveError('%s is not an archive' % name)
 
         if flag:
-            raise ValueError('Cannot open compressed archive %s in place' %
-                    name)
+            raise ValueError('Cannot open compressed archive %s in place' % name)
         return CArchiveReader(self.path, self.pkg_start + dpos, dlen)
